@@ -1,3 +1,4 @@
+'use strict'
 
 var express = require('../')
 
@@ -5,10 +6,6 @@ describe('app.listen()', function(){
   it('should wrap with an HTTP server', function(done){
     var app = express();
 
-    app.del('/tobi', function(req, res){
-      res.end('deleted tobi!');
-    });
-
     var server = app.listen(9999, function(){
       server.close();
       done();
